@@ -160,7 +160,7 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 }
 
 uint16_t
-ptp_usb_getdata (PTPParams* params, PTPContainer* ptp,
+ptp_usb_getdata (PTPParams* params, PTPContainer* ptp,  unsigned int *getlen, 
 		unsigned char **data)
 {
 	static uint16_t ret;
@@ -171,7 +171,6 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp,
 	if (*data!=NULL) return PTP_ERROR_BADPARAM;
 #endif
 	do {
-		static uint32_t len;
 		/* read first(?) part of data */
 		ret=params->read_func((unsigned char *)&usbdata,
 				sizeof(usbdata), params->data);
@@ -179,7 +178,8 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp,
 			ret = PTP_ERROR_IO;
 			break;
 		} else
-		if (dtoh16(usbdata.type)!=PTP_USB_CONTAINER_DATA) {
+		if (dtoh16(usbdata.type)!=PTP_USB_CONTAINER_DATA
+			&& dtoh16(usbdata.type)!=PTP_USB_CONTAINER_RESPONSE) {
 			ret = PTP_ERROR_DATA_EXPECTED;
 			break;
 		} else
@@ -188,19 +188,19 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp,
 			break;
 		}
 		/* evaluate data length */
-		len=dtoh32(usbdata.length)-PTP_USB_BULK_HDR_LEN;
+		*getlen=dtoh32(usbdata.length)-PTP_USB_BULK_HDR_LEN;
 		/* allocate memory for data if not allocated already */
-		if (*data==NULL) *data=calloc(1,len);
+		if (*data==NULL) *data=calloc(1,*getlen);
 		/* copy first part of data to 'data' */
 		memcpy(*data,usbdata.payload.data,
-			PTP_USB_BULK_PAYLOAD_LEN<len?
-			PTP_USB_BULK_PAYLOAD_LEN:len);
+			PTP_USB_BULK_PAYLOAD_LEN<*getlen?
+			PTP_USB_BULK_PAYLOAD_LEN:*getlen);
 		/* is that all of data? */
-		if (len+PTP_USB_BULK_HDR_LEN<=sizeof(usbdata)) break;
+		if (*getlen+PTP_USB_BULK_HDR_LEN<=sizeof(usbdata)) break;
 		/* if not finaly read the rest of it */
 		ret=params->read_func(((unsigned char *)(*data))+
 					PTP_USB_BULK_PAYLOAD_LEN,
-					len-PTP_USB_BULK_PAYLOAD_LEN,
+					*getlen-PTP_USB_BULK_PAYLOAD_LEN,
 					params->data);
 		if (ret!=PTP_RC_OK) {
 			ret = PTP_ERROR_IO;
@@ -256,12 +256,6 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 
 /* major PTP functions */
 
-/* Transaction data phase description */
-#define PTP_DP_NODATA		0x0000	/* no data phase */
-#define PTP_DP_SENDDATA		0x0001	/* sending data */
-#define PTP_DP_GETDATA		0x0002	/* receiving data */
-#define PTP_DP_DATA_MASK	0x00ff	/* data phase mask */
-
 /* Number of PTP Request phase parameters */
 #define PTP_RQ_PARAM0		0x0000	/* zero parameters */
 #define PTP_RQ_PARAM1		0x0100	/* one parameter */
@@ -297,7 +291,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
  * Upon success PTPContainer* ptp contains PTP Response Phase container with
  * all fields filled in.
  **/
-static uint16_t
+ uint16_t
 ptp_transaction (PTPParams* params, PTPContainer* ptp, 
 			uint16_t flags, unsigned int sendlen, char** data)
 {
@@ -315,8 +309,13 @@ ptp_transaction (PTPParams* params, PTPContainer* ptp,
 				(unsigned char*)*data, sendlen));
 			break;
 		case PTP_DP_GETDATA:
+			{
+			unsigned int getlen;
 			CHECK_PTP_RC(params->getdata_func(params, ptp,
+				    sendlen?(unsigned int *)(((uint64_t)(&getlen)&0xffffffff00000000)|sendlen):
+				    &getlen,
 				(unsigned char**)data));
+			}
 			break;
 		case PTP_DP_NODATA:
 			break;
@@ -836,6 +835,39 @@ ptp_setdevicepropvalue (PTPParams* params, uint16_t propcode,
 	return ret;
 }
 
+uint16_t
+ptp_sendgenericrequest (PTPParams* params, uint16_t reqcode,
+						uint32_t* reqparams, char** data, uint32_t direction, long sendlen)
+{
+	PTPContainer ptp;
+	uint16_t ret=0;
+	char *dpv=NULL;
+	
+	if (direction == PTP_DP_GETDATA)
+		*data = NULL;
+
+	ptp_debug(params, "PTP: Sending generic Request 0x%04x", reqcode);
+
+	PTP_CNT_INIT(ptp);
+	ptp.Code=reqcode;
+	ptp.Nparam=0;
+	// carfull here, do not only rely on the nullity of a value,
+	// because the next parameter might be not null
+	if((ptp.Param1=reqparams[0]) != 0)
+		ptp.Nparam = 1;
+	if((ptp.Param2=reqparams[1]) != 0)
+		ptp.Nparam = 2;
+	if((ptp.Param3=reqparams[2]) != 0)
+		ptp.Nparam = 3;
+	if((ptp.Param4=reqparams[3]) != 0)
+		ptp.Nparam = 4;
+	if((ptp.Param5=reqparams[4]) != 0)
+		ptp.Nparam = 5;
+
+	ret=ptp_transaction(params, &ptp, direction, sendlen, data);
+	return ret;
+}
+
 /**
  * ptp_ek_sendfileobjectinfo:
  * params:	PTPParams*
