@@ -74,6 +74,8 @@ struct _PTPUSBBulkContainer {
 };
 typedef struct _PTPUSBBulkContainer PTPUSBBulkContainer;
 
+#define PTP_USB_INT_PACKET_LEN	8
+
 /* PTP USB Asynchronous Event Interrupt Data Format */
 struct _PTPUSBEventContainer {
 	uint32_t length;
@@ -108,6 +110,12 @@ typedef struct _PTPUSBEventContainer PTPUSBEventContainer;
 #define PTP_VENDOR_NIKON		0x0000000A
 #define PTP_VENDOR_CANON		0x0000000B
 
+/* Transaction data phase description */
+#define PTP_DP_NODATA		0x0000	/* no data phase */
+#define PTP_DP_SENDDATA		0x0001	/* sending data */
+#define PTP_DP_GETDATA		0x0002	/* receiving data */
+#define PTP_DP_DATA_MASK	0x00ff	/* data phase mask */
+
 /* Operation Codes */
 
 #define PTP_OC_Undefined                0x1000
@@ -157,6 +165,24 @@ typedef struct _PTPUSBEventContainer PTPUSBEventContainer;
 #define PTP_OC_CANON_GetViewfinderImage	0x901d
 #define PTP_OC_CANON_GetChanges		0x9020
 #define PTP_OC_CANON_GetFolderEntries	0x9021
+/* MTP extension Operation Codes */
+#define	PTP_OC_MTP_GetObjectPropsSupported	0x9801
+#define	PTP_OC_MTP_GetObjectPropDesc		0x9802
+#define	PTP_OC_MTP_GetObjectPropValue		0x9803
+#define	PTP_OC_MTP_SetObjectPropValue		0x9804
+#define	PTP_OC_MTP_GetObjPropList	0x9805
+#define	PTP_OC_MTP_SetObjPropList	0x9806
+#define	PTP_OC_MTP_GetInterdependendPropDesc	0x9807
+#define	PTP_OC_MTP_SendObjectPropList		0x9808
+#define	PTP_OC_MTP_GetObjectReferences		0x9810
+#define	PTP_OC_MTP_SetObjectReferences		0x9811
+#define	PTP_OC_MTP_UpdateDeviceFirmware		0x9812
+#define	PTP_OC_MTP_Skip			0x9820
+/* Nikon extensiion Operation Codes */
+#define PTP_OC_NIKON_DirectCapture	0x90C0
+#define PTP_OC_NIKON_SetControlMode	0x90C2
+#define PTP_OC_NIKON_CheckEvent		0x90C7
+#define PTP_OC_NIKON_KeepAlive		0x90C8
 
 /* Proprietary vendor extension operations mask */
 #define PTP_OC_EXTENSION_MASK		0xF000
@@ -202,6 +228,9 @@ typedef struct _PTPUSBEventContainer PTPUSBEventContainer;
 #define PTP_RC_EK_FilenameConflicts	0xA002
 #define PTP_RC_EK_FilenameInvalid	0xA003
 
+/* NIKON extension Response Codes */
+#define PTP_RC_NIKON_PropertyReadOnly	0xA005
+
 /* Proprietary vendor extension response code mask */
 #define PTP_RC_EXTENSION_MASK		0xF000
 #define PTP_RC_EXTENSION		0xA000
@@ -234,6 +263,10 @@ typedef struct _PTPUSBEventContainer PTPUSBEventContainer;
 #define PTP_EC_CANON_RequestObjectTransfer	0xC009
 #define PTP_EC_CANON_CameraModeChanged	0xC00C
 
+/* Nikon extension Event Codes */
+#define PTP_EC_NIKON_ObjectReady	0xC101
+#define PTP_EC_NIKON_CaptureOverflow	0xC102
+
 /* PTP device info structure (returned by GetDevInfo) */
 
 struct _PTPDeviceInfo {
@@ -540,6 +573,7 @@ typedef struct _PTPCANONFolderEntry PTPCANONFolderEntry;
 #define PTP_DPC_CANON_CameraModel	0xD032
 #define PTP_DPC_CANON_CameraOwner	0xD033
 #define PTP_DPC_CANON_UnixTime		0xD034
+#define PTP_DPC_CANON_ViewfinderOutput	0xD036
 #define PTP_DPC_CANON_RealImageWidth	0xD039
 #define PTP_DPC_CANON_PhotoEffect	0xD040
 #define PTP_DPC_CANON_AssistLight	0xD041
@@ -555,17 +589,17 @@ typedef struct _PTPCANONFolderEntry PTPCANONFolderEntry;
 #define PTP_DPC_NIKON_WhiteBalanceAutoBias		0xD017
 #define PTP_DPC_NIKON_WhiteBalanceTungstenBias		0xD018
 #define PTP_DPC_NIKON_WhiteBalanceFlourescentBias	0xD019
-#define PTP_DPC_NIKON_WhiteBalanceDaylightBias		0xD01a
-#define PTP_DPC_NIKON_WhiteBalanceFlashBias		0xD01b
-#define PTP_DPC_NIKON_WhiteBalanceCloudyBias		0xD01c
-#define PTP_DPC_NIKON_WhiteBalanceShadeBias		0xD01d
-#define PTP_DPC_NIKON_WhiteBalanceColourTemperature	0xD01e
-#define PTP_DPC_NIKON_ImageSharpening			0xD02a
-#define PTP_DPC_NIKON_ToneCompensation			0xD02b
-#define PTP_DPC_NIKON_ColourMode			0xD02c
-#define PTP_DPC_NIKON_HueAdjustment			0xD02d
-#define PTP_DPC_NIKON_NonCPULensDataFocalLength		0xD02e
-#define PTP_DPC_NIKON_NonCPULensDataMaximumAperture	0xD02f
+#define PTP_DPC_NIKON_WhiteBalanceDaylightBias		0xD01A
+#define PTP_DPC_NIKON_WhiteBalanceFlashBias		0xD01B
+#define PTP_DPC_NIKON_WhiteBalanceCloudyBias		0xD01C
+#define PTP_DPC_NIKON_WhiteBalanceShadeBias		0xD01D
+#define PTP_DPC_NIKON_WhiteBalanceColorTemperature	0xD01E
+#define PTP_DPC_NIKON_ImageSharpening			0xD02A
+#define PTP_DPC_NIKON_ToneCompensation			0xD02B
+#define PTP_DPC_NIKON_ColorMode				0xD02C
+#define PTP_DPC_NIKON_HueAdjustment			0xD02D
+#define PTP_DPC_NIKON_NonCPULensDataFocalLength		0xD02E
+#define PTP_DPC_NIKON_NonCPULensDataMaximumAperture	0xD02F
 #define PTP_DPC_NIKON_CSMMenuBankSelect			0xD040
 #define PTP_DPC_NIKON_MenuBankNameA			0xD041
 #define PTP_DPC_NIKON_MenuBankNameB			0xD042
@@ -573,85 +607,107 @@ typedef struct _PTPCANONFolderEntry PTPCANONFolderEntry;
 #define PTP_DPC_NIKON_MenuBankNameD			0xD044
 #define PTP_DPC_NIKON_A1AFCModePriority			0xD048
 #define PTP_DPC_NIKON_A2AFSModePriority			0xD049
-#define PTP_DPC_NIKON_A3GroupDynamicAF			0xD04a
-#define PTP_DPC_NIKON_A4AFActivation			0xD04b
-#define PTP_DPC_NIKON_A5FocusAreaIllumManualFocus	0xD04c
-#define PTP_DPC_NIKON_FocusAreaIllumContinuous		0xD04d
-#define PTP_DPC_NIKON_FocusAreaIllumWhenSelected 	0xD04e
-#define PTP_DPC_NIKON_A6FocusArea			0xD04f
+#define PTP_DPC_NIKON_A3GroupDynamicAF			0xD04A
+#define PTP_DPC_NIKON_A4AFActivation			0xD04B
+#define PTP_DPC_NIKON_A5FocusAreaIllumManualFocus	0xD04C
+#define PTP_DPC_NIKON_FocusAreaIllumContinuous		0xD04D
+#define PTP_DPC_NIKON_FocusAreaIllumWhenSelected 	0xD04E
+#define PTP_DPC_NIKON_FocusAreaWrap			0xD04F
 #define PTP_DPC_NIKON_A7VerticalAFON			0xD050
-#define PTP_DPC_NIKON_B1ISOAuto				0xD054
+#define PTP_DPC_NIKON_ISOAuto				0xD054
 #define PTP_DPC_NIKON_B2ISOStep				0xD055
-#define PTP_DPC_NIKON_B3EVStep				0xD056
+#define PTP_DPC_NIKON_EVStep				0xD056
 #define PTP_DPC_NIKON_B4ExposureCompEv			0xD057
-#define PTP_DPC_NIKON_B5ExposureComp			0xD058
-#define PTP_DPC_NIKON_B6CenterWeightArea		0xD059
-#define PTP_DPC_NIKON_C1AELock				0xD05e
-#define PTP_DPC_NIKON_C2AELAFL				0xD05f
-#define PTP_DPC_NIKON_C3AutoMeterOff			0xD062
-#define PTP_DPC_NIKON_C4SelfTimer			0xD063
-#define PTP_DPC_NIKON_C5MonitorOff			0xD064
+#define PTP_DPC_NIKON_ExposureCompensation		0xD058
+#define PTP_DPC_NIKON_CenterWeightArea			0xD059
+#define PTP_DPC_NIKON_AELockMode			0xD05E
+#define PTP_DPC_NIKON_AELAFLMode			0xD05F
+#define PTP_DPC_NIKON_MeterOff				0xD062
+#define PTP_DPC_NIKON_SelfTimer				0xD063
+#define PTP_DPC_NIKON_MonitorOff			0xD064
 #define PTP_DPC_NIKON_D1ShootingSpeed			0xD068
 #define PTP_DPC_NIKON_D2MaximumShots			0xD069
-#define PTP_DPC_NIKON_D3ExpDelayMode			0xD06a
-#define PTP_DPC_NIKON_D4LongExposureNoiseReduction	0xD06b
-#define PTP_DPC_NIKON_D5FileNumberSequence		0xD06c
-#define PTP_DPC_NIKON_D6ControlPanelFinderRearControl	0xD06d
-#define PTP_DPC_NIKON_ControlPanelFinderViewfinder	0xD06e
-#define PTP_DPC_NIKON_D7Illumination			0xD06f
+#define PTP_DPC_NIKON_D3ExpDelayMode			0xD06A
+#define PTP_DPC_NIKON_LongExposureNoiseReduction	0xD06B
+#define PTP_DPC_NIKON_FileNumberSequence		0xD06C
+#define PTP_DPC_NIKON_D6ControlPanelFinderRearControl	0xD06D
+#define PTP_DPC_NIKON_ControlPanelFinderViewfinder	0xD06E
+#define PTP_DPC_NIKON_D7Illumination			0xD06F
 #define PTP_DPC_NIKON_E1FlashSyncSpeed			0xD074
-#define PTP_DPC_NIKON_E2FlashShutterSpeed		0xD075
+#define PTP_DPC_NIKON_FlashShutterSpeed			0xD075
 #define PTP_DPC_NIKON_E3AAFlashMode			0xD076
 #define PTP_DPC_NIKON_E4ModelingFlash			0xD077
-#define PTP_DPC_NIKON_E5AutoBracketSet			0xD078
+#define PTP_DPC_NIKON_BracketSet			0xD078
 #define PTP_DPC_NIKON_E6ManualModeBracketing		0xD079	
-#define PTP_DPC_NIKON_E7AutoBracketOrder		0xD07a
-#define PTP_DPC_NIKON_E8AutoBracketSelection		0xD07b
+#define PTP_DPC_NIKON_BracketOrder			0xD07A
+#define PTP_DPC_NIKON_E8AutoBracketSelection		0xD07B
+#define PTP_DPC_NIKON_BracketingSet			0xD07C
+
 #define PTP_DPC_NIKON_F1CenterButtonShootingMode	0xD080
 #define PTP_DPC_NIKON_CenterButtonPlaybackMode		0xD081
 #define PTP_DPC_NIKON_F2Multiselector			0xD082
 #define PTP_DPC_NIKON_F3PhotoInfoPlayback		0xD083
 #define PTP_DPC_NIKON_F4AssignFuncButton		0xD084
 #define PTP_DPC_NIKON_F5CustomizeCommDials		0xD085
-#define PTP_DPC_NIKON_ChangeMainSub			0xD086
+#define PTP_DPC_NIKON_ReverseCommandDial		0xD086
 #define PTP_DPC_NIKON_ApertureSetting			0xD087
 #define PTP_DPC_NIKON_MenusAndPlayback			0xD088
 #define PTP_DPC_NIKON_F6ButtonsAndDials			0xD089
-#define PTP_DPC_NIKON_F7NoCFCard			0xD08a
+#define PTP_DPC_NIKON_NoCFCard				0xD08A
 #define PTP_DPC_NIKON_ImageCommentString		0xD090
-#define PTP_DPC_NIKON_ImageCommentAttached		0xD091
-#define PTP_DPC_NIKON_AutoImageRotation			0xD092
-#define PTP_DPC_NIKON_ExposureBracketingOnOff		0xD0c0
-#define PTP_DPC_NIKON_ExposureBracketingIntervalDist	0xD0c1
-#define PTP_DPC_NIKON_ExposureBracketingNumBracketPlace	0xD0c2
-#define PTP_DPC_NIKON_AutofocusLCDTopMode2		0xD107
-#define PTP_DPC_NIKON_AutofocusLCDTopMode3AndMode4	0xD108
-#define PTP_DPC_NIKON_LightMeter			0xD10a
-#define PTP_DPC_NIKON_ExposureApertureLock		0xD111
-#define PTP_DPC_NIKON_MaximumShots			0xD103
-#define PTP_DPC_NIKON_Beep                              0xD160
-#define PTP_DPC_NIKON_AFC                               0xD161
-#define PTP_DPC_NIKON_AFLampOff                         0xD163
-#define PTP_DPC_NIKON_PADVPMode                         0xD164
-#define PTP_DPC_NIKON_ReviewOff                         0xD165
-#define PTP_DPC_NIKON_GridDisplay                       0xD16c
-#define PTP_DPC_NIKON_AFAreaIllumination                0xD166
-#define PTP_DPC_NIKON_FlashMode                         0xD167
-#define PTP_DPC_NIKON_FlashPower                        0xD16d
-#define PTP_DPC_NIKON_FlashSignOff                      0xD169
-#define PTP_DPC_NIKON_RemoteTimeout                     0xD16B
-#define PTP_DPC_NIKON_LowLight                          0xD1B0
-#define PTP_DPC_NIKON_ImageCommentString                0xD090
-#define PTP_DPC_NIKON_FlashOpen                         0xD1C0
-#define PTP_DPC_NIKON_FlashCharged                      0xD1C1
+#define PTP_DPC_NIKON_ImageCommentAttach		0xD091
+#define PTP_DPC_NIKON_ImageRotation			0xD092
+#define PTP_DPC_NIKON_Bracketing			0xD0C0
+#define PTP_DPC_NIKON_ExposureBracketingIntervalDist	0xD0C1
+#define PTP_DPC_NIKON_BracketingProgram			0xD0C2
+#define PTP_DPC_NIKON_WhiteBalanceBracketStep		0xD0C4
 #define PTP_DPC_NIKON_LensID                            0xD0E0
 #define PTP_DPC_NIKON_FocalLengthMin                    0xD0E3
 #define PTP_DPC_NIKON_FocalLengthMax                    0xD0E4
 #define PTP_DPC_NIKON_MaxApAtMinFocalLength             0xD0E5
 #define PTP_DPC_NIKON_MaxApAtMaxFocalLength             0xD0E6
+#define PTP_DPC_NIKON_ExposureTime			0xD100
+#define PTP_DPC_NIKON_ACPower				0xD101
+#define PTP_DPC_NIKON_MaximumShots			0xD103
+#define PTP_DPC_NIKON_AFLLock				0xD104
+#define PTP_DPC_NIKON_AutoExposureLock			0xD105
+#define PTP_DPC_NIKON_AutoFocusLock			0xD106
+#define PTP_DPC_NIKON_AutofocusLCDTopMode2		0xD107
+#define PTP_DPC_NIKON_AutofocusArea			0xD108
+#define PTP_DPC_NIKON_LightMeter			0xD10A
+#define PTP_DPC_NIKON_CameraOrientation			0xD10E
+#define PTP_DPC_NIKON_ExposureApertureLock		0xD111
+#define PTP_DPC_NIKON_BeepOff				0xD160
+#define PTP_DPC_NIKON_AutofocusMode			0xD161
+#define PTP_DPC_NIKON_AFAssist				0xD163
+#define PTP_DPC_NIKON_PADVPMode                         0xD164
+#define PTP_DPC_NIKON_ImageReview			0xD165
+#define PTP_DPC_NIKON_AFAreaIllumination                0xD166
+#define PTP_DPC_NIKON_FlashMode                         0xD167
+#define PTP_DPC_NIKON_FlashCommanderMode		0xD168
+#define PTP_DPC_NIKON_FlashSign				0xD169
+#define PTP_DPC_NIKON_GridDisplay                       0xD16C
+#define PTP_DPC_NIKON_FlashModeManualPower		0xD16D
+#define PTP_DPC_NIKON_FlashModeCommanderPower		0xD16E
+#define PTP_DPC_NIKON_RemoteTimeout                     0xD16B
+#define PTP_DPC_NIKON_GridDisplay			0xD16C
+#define PTP_DPC_NIKON_BracketingIncrement		0xD190
+#define PTP_DPC_NIKON_LowLight                          0xD1B0
+#define PTP_DPC_NIKON_FlashOpen                         0xD1C0
+#define PTP_DPC_NIKON_FlashCharged                      0xD1C1
 #define PTP_DPC_NIKON_FlashExposureCompensation         0xD126
-#define PTP_DPC_NIKON_ExtendedCSMMenu		        0xD180
-#define PTP_DPC_NIKON_OptimiseImage		        0xD140
+#define PTP_DPC_NIKON_CSMMenu			        0xD180
+#define PTP_DPC_NIKON_OptimizeImage		        0xD140
+#define PTP_DPC_NIKON_Saturation		        0xD142
+/* MTP extension device property codes */
+#define	PTP_DPC_MTP_SynchronizationPartner	0xD401
+#define	PTP_DPC_MTP_DeviceFriendlyName		0xD402
+#define	PTP_DPC_MTP_Volume			0xD403
+#define	PTP_DPC_MTP_SupportedFormatsOrdered	0xD404
+#define	PTP_DPC_MTP_DeviceIcon			0xD405
+#define	PTP_DPC_MTP_PlaybackRate		0xD410
+#define	PTP_DPC_MTP_PlaybackObject		0xD411
+#define	PTP_DPC_MTP_PlaybackContainerIndex	0xD412
 
 /* Device Property Form Flag */
 
@@ -681,6 +737,7 @@ typedef uint16_t (* PTPIOSendData)	(PTPParams* params, PTPContainer* ptp,
 					unsigned char *data, unsigned int size);
 typedef uint16_t (* PTPIOGetResp)	(PTPParams* params, PTPContainer* resp);
 typedef uint16_t (* PTPIOGetData)	(PTPParams* params, PTPContainer* ptp,
+					uint32_t *getlen,
 					unsigned char **data);
 /* debug functions */
 typedef void (* PTPErrorFunc) (void *data, const char *format, va_list args);
@@ -727,8 +784,8 @@ uint16_t ptp_usb_sendreq	(PTPParams* params, PTPContainer* req);
 uint16_t ptp_usb_senddata	(PTPParams* params, PTPContainer* ptp,
 				unsigned char *data, unsigned int size);
 uint16_t ptp_usb_getresp	(PTPParams* params, PTPContainer* resp);
-uint16_t ptp_usb_getdata	(PTPParams* params, PTPContainer* ptp, 
-				unsigned char **data);
+uint16_t ptp_usb_getdata	(PTPParams* params, PTPContainer* ptp,  
+				unsigned int *getlen, unsigned char **data);
 uint16_t ptp_usb_event_check	(PTPParams* params, PTPContainer* event);
 uint16_t ptp_usb_event_wait		(PTPParams* params, PTPContainer* event);
 
@@ -771,7 +828,12 @@ uint16_t ptp_getdevicepropdesc	(PTPParams* params, uint16_t propcode,
 uint16_t ptp_getdevicepropvalue	(PTPParams* params, uint16_t propcode,
 				void** value, uint16_t datatype);
 uint16_t ptp_setdevicepropvalue (PTPParams* params, uint16_t propcode,
-                        	void* value, uint16_t datatype);
+				void* value, uint16_t datatype);
+
+
+uint16_t ptp_sendgenericrequest (PTPParams* params, uint16_t reqcode,
+				uint32_t* reqparams, char** data,
+				uint32_t direction, long sendlen);
 
 
 uint16_t ptp_ek_sendfileobjectinfo (PTPParams* params, uint32_t* store,
@@ -811,6 +873,13 @@ uint16_t ptp_canon_getfolderentries (PTPParams* params, uint32_t store,
 				PTPCANONFolderEntry** entries,
 				uint32_t* entnum);
 
+/* Nikon extensions */
+uint16_t ptp_nikon_setcontrolmode (PTPParams* params, uint32_t mode);
+uint16_t ptp_nikon_directcapture (PTPParams* params, uint32_t unknown);
+uint16_t ptp_nikon_checkevent (PTPParams* params,
+				PTPUSBEventContainer** event, uint16_t* evnum);
+uint16_t ptp_nikon_keepalive (PTPParams* params);
+
 
 /* Non PTP protocol functions */
 int ptp_operation_issupported	(PTPParams* params, uint16_t operation);
@@ -819,10 +888,24 @@ int ptp_property_issupported	(PTPParams* params, uint16_t property);
 void ptp_free_devicepropdesc	(PTPDevicePropDesc* dpd);
 void ptp_perror			(PTPParams* params, uint16_t error);
 const char*
+ptp_get_datatype_name		(PTPParams* params, uint16_t dt);
+const char*
 ptp_get_operation_name		(PTPParams* params, uint16_t oc);
 const char*
-ptp_get_property_name		(PTPParams* params, uint16_t dpc);
+ptp_prop_getname		(PTPParams* params, uint16_t dpc);
 
+/* Properties handling functions */
+const char* ptp_prop_getdesc	(PTPParams* params, PTPDevicePropDesc *dpd,
+					void *val);
+const char* ptp_prop_getdescbystring
+				(PTPParams* params,PTPDevicePropDesc *dpd,
+				const char *strval);
+
+const char * ptp_prop_tostr	(PTPParams* params, PTPDevicePropDesc *dpd,
+					void *val);
+uint16_t ptp_prop_getcodebyname	(PTPParams* params, char* propname);
+const char* ptp_prop_getvalbyname
+				(PTPParams* params, char* name, uint16_t dpc);
 
 
 #endif /* __PTP_H__ */
