@@ -112,11 +112,12 @@ help()
 	"  --dev=DEV-NUMBER             USB assigned device number\n"
 	"  -r, --reset                  Reset the device\n"
 	"  -l, --list-devices           List all PTP devices\n"
+	"                               \n"
 	"  -i, --info                   Show device info\n"
-	"  -o, --list-operations        List supported operations\n"
-	"  -p, --list-properties        List all PTP device properties\n"
+	"  -o, --list-operations        List all supported operations\n"
+	"  -p, --list-properties        List all supported device properties\n"
 	"                               "
-				"(e.g. focus mode, focus distance, etc.)\n"
+					"(e.g. focus mode, focus distance, etc.)\n"
 	"  -s, --show-property=NUMBER   Display property details "
 					"(or set its value,\n"
 	"                               if used in conjunction with --val)\n"
@@ -126,23 +127,72 @@ help()
 	"                               string or numeric for --set)\n"
 	"  --show-all-properties        Show all properties values\n"
 	"  --show-unknown-properties    Show unknown properties values\n"
+	"                               \n"
 	"  -L, --list-files             List all files\n"
 	"  -g, --get-file=HANDLE        Get file by given handler\n"
 	"  -G, --get-all-files          Get all files\n"
-	"  --overwrite                  Force file overwrite while saving"
-					"to disk\n"
+	"                               \n"
 	"  -d, --delete-object=HANDLE   Delete object (file) by given handle\n"
 	"  -D, --delete-all-files       Delete all files form camera\n"
+	"                               \n"
 	"  -c, --capture                Initiate capture\n"
+	"  --loop-capture=N             Perform N times capture/get/delete sequence\n"
+	"  --interval=N                 Set loop capture interval in seconds\n"
+	"                               Following Nikon commands are EXPERIMENTAL!\n"
 	"  --nikon-ic, --nic            Initiate Nikon Direct Capture (no download!)\n"
 	"  --nikon-dc, --ndc            Initiate Nikon Direct Capture and download\n"
-	"  --loop-capture=N             Perform N times capture/get/delete\n"
+	"                               \n"
+	"  -R reqCode[,P1,P2,P3,P4,P5,d] Send a raw generic request with parameters\n"
+	"                               parameters Pn are optionnal and can be set to 0\n"
+	"                               if not used (params values must be hexadecimal)\n"
+	"                               d, is the data direction (r=read, n=no data, \n"
+	"                               filename or value to send in hexadecimal).\n"
+	"                               The default data direction is 'r'.\n"
+	"                               If you want to send data, give the file name\n"
+	"                               containing data to send, or, if the data is short,\n"
+	"                               you can directly write the value in hexadecimal.\n"
+	"                               The result data and response will be displayed\n"
+	"                      example: ptpcam -R 0x1015,0x5003,0,0,0,0,r\n"
+	"                               ptpcam -R 0x1016,0x5003,0,0,0,0,image_size.bin\n"
+	"                               ptpcam -R 0x1016,0x501b,0,0,0,0,0x88130000\n"
+	"                               ptpcam -R 0x100e,0,0,0,0,0,n\n"
+	"                               \n"
+	"  --overwrite                  Force file overwrite while saving"
+					"to disk\n"
 	"  -f, --force                  Talk to non PTP devices\n"
 	"  -v, --verbose                Be verbose (print more debug)\n"
 	"  -h, --help                   Print this help message\n"
 	"\n");
 }
 
+void
+display_hexdump(char *data, size_t size)
+{
+	uint i=0;
+	char buffer[50];
+	char charBuf[17];
+	
+	memset((void*)buffer, 0, 49);
+	memset((void*)charBuf, 0, 17);
+	for (; i < size; ++i)
+	{
+		snprintf(&(buffer[(i%16) * 3]) , 4, "%02x ", *(data+i)&0xff);
+		if ((data[i] >= ' ' && data[i] <= '^') ||
+			(data[i] >= 'a' && data[i] <= '~')) // printable characters
+			charBuf[i%16] = data[i];
+		else
+			charBuf[i%16] = '.';
+		if ((i % 16) == 15 || i == (size - 1))
+		{
+			charBuf[(i%16)+1] = '\0';
+			buffer[((i%16) * 3) + 2] = '\0';
+			printf("%-48s- %-16s\n", buffer, charBuf);
+			memset((void*)buffer, 0, 49);
+			memset((void*)charBuf, 0, 17);
+		}
+	}
+}
+
 void
 ptpcam_siginthandler(int signum)
 {
@@ -245,7 +295,6 @@ ptpcam_error (void *data, const char *format, va_list args);
 void
 ptpcam_error (void *data, const char *format, va_list args)
 {
-/*	if (!verbose) return; */
 	vfprintf (stderr, format, args);
 	fprintf (stderr,"\n");
 	fflush(stderr);
@@ -586,8 +635,12 @@ out:
 	close_camera(&ptp_usb, &params, dev);
 }
 
+static void sig_alrm(int signo)
+{
+}
+
 void
-loop_capture (int busn, int devn, short force, int n,  int overwrite)
+loop_capture (int busn, int devn, short force, int n, int interval, int overwrite)
 {
 	PTPParams params;
 	PTP_USB ptp_usb;
@@ -599,6 +652,11 @@ loop_capture (int busn, int devn, short force, int n,  int overwrite)
 	char *image;
 	int ret;
 	char *filename;
+	time_t start_time;
+	time_t rest_time;
+
+	// Catch the SIGALARM
+	signal(SIGALRM, sig_alrm);
 
 	if (open_camera(busn, devn, force, &ptp_usb, &params, &dev)<0)
 		return;
@@ -608,10 +666,10 @@ loop_capture (int busn, int devn, short force, int n,  int overwrite)
 
 	printf("Camera: %s\n",params.deviceinfo.Model);
 
-
 	/* local loop */
 	while (n>0) {
 		/* capture */
+		time(&start_time);
 		printf("\nInitiating captue...\n");
 		CR(ptp_initiatecapture (&params, 0x0, 0),"Could not capture\n");
 		n--;
@@ -686,6 +744,12 @@ download:
 			printf("Object 0x%08lx (%s) deleted.\n",(long unsigned) handle, oi.Filename);
 		}
 out:
+		rest_time=interval-(time(NULL)-start_time);
+		if (rest_time>0 && n>0) {
+		    printf("Sleeping for remaining %u seconds.\n",rest_time);
+		    alarm(rest_time);
+		    pause();
+		}
 		;
 	}
 err:
@@ -808,10 +872,10 @@ nikon_direct_capture (int busn, int devn, short force, char* filename,int overwr
 	if (BurstNumber>1) printf("Capturing %i frames in burst.\n",BurstNumber);
 
 	/* sleep in case of exposure longer than 1/100 */
-	if (ExposureTime>10) {
-	    printf ("sleeping %i miliseconds\n", ExposureTime);
-	    usleep (ExposureTime*1000);
-	}
+	//if (ExposureTime>10) {
+	    printf ("sleeping %i miliseconds\n", 500+ExposureTime);
+	    usleep (ExposureTime*1000+500000);
+	//}
 
 	while (BurstNumber>0) {
 
@@ -1239,6 +1303,86 @@ get_all_files (int busn, int devn, short force, int overwrite)
 	close_camera(&ptp_usb, &params, dev);
 }
 
+void
+send_generic_request (int busn, int devn, uint16_t reqCode, uint32_t *reqParams, uint32_t direction, char *data_file)
+{
+	PTPParams params;
+	PTP_USB ptp_usb;
+	struct usb_device *dev;
+	char *data = NULL;
+	long fsize = 0;
+
+	if (direction == PTP_DP_SENDDATA) { // read data from file
+		if (strncmp(data_file, "0x", 2) == 0) {
+		    uint len = strlen(data_file);
+			char num[3];
+			uint i;
+			data = (char*)calloc(1,len / 2);
+
+			num[2] = 0;
+			for (i = 2; i < len ; i += 2) {
+				num[1] = data_file[i];
+				if (i < len-1) {
+					num[0] = data_file[i];
+				    num[1] = data_file[i+1];
+				}
+				else {
+				    num[0] = data_file[i];
+					num[1] = 0;
+				}
+				data[fsize] = (char)strtol(num, NULL, 16);
+				++fsize;
+			}
+		}
+		else {
+		    FILE *f = fopen(data_file, "r");
+			if (f) {
+				fseek(f, 0, SEEK_END);
+				fsize = ftell(f);
+				fseek(f, 0, SEEK_SET);
+				data = (char*)calloc(1,fsize + 1);
+				if (fread(data, 1, fsize, f) != fsize) {
+					fprintf(stderr, "PTP: ERROR: can't read data to send from file '%s'\n", data_file);
+					free (data);
+					return;
+				}
+				else {
+					printf("--- data to send ---\n");
+					display_hexdump(data, fsize);
+					printf("--------------------\n");
+				}
+			} else { // error no data to send
+				fprintf(stderr, "PTP: ERROR: file not found '%s'\n", data_file);
+				return;
+			}
+		}
+	}
+
+	if (open_camera(busn, devn, 0, &ptp_usb, &params, &dev)<0)
+		return;
+	printf("Camera: %s\n",params.deviceinfo.Model);
+
+	printf("Sending generic request: reqCode=0x%04x, params=[0x%08x,0x%08x,0x%08x,0x%08x,0x%08x]\n",
+			(uint)reqCode, reqParams[0], reqParams[1], reqParams[2], reqParams[3], reqParams[4]);
+	uint16_t result=ptp_sendgenericrequest (&params, reqCode, reqParams, &data, direction, fsize);
+	if((result)!=PTP_RC_OK) {
+		ptp_perror(&params,result);
+		if (result > 0x2000)
+			fprintf(stderr,"PTP: ERROR: response 0x%04x\n", result);
+	} else {
+	    if (data != NULL && direction == PTP_DP_GETDATA) {
+		    display_hexdump(data, malloc_usable_size ((void*)data));
+			free(data);
+		}
+		printf("PTP: response OK\n");
+	}
+	if (data != NULL && direction == PTP_DP_SENDDATA) {
+		free(data);
+	}
+	close_camera(&ptp_usb, &params, dev);
+	return;
+	
+}
 
 void
 list_operations (int busn, int devn, short force)
@@ -1283,11 +1427,6 @@ list_properties (int busn, int devn, short force)
 
 	if (open_camera(busn, devn, force, &ptp_usb, &params, &dev)<0)
 		return;
-/* XXX */
-#if 0
-	CR(ptp_nikon_setcontrolmode(&params, 0x01),
-		"Unable to set Nikon PC controll mode\n");
-#endif
 	printf("Camera: %s\n",params.deviceinfo.Model);
 	for (i=0; i<params.deviceinfo.DevicePropertiesSupported_len;i++){
 		propname=ptp_prop_getname(&params,
@@ -1384,9 +1523,9 @@ set_property (PTPParams* params,
 	return 0;
 }
 void
-getset_property_internal (PTPParams* params, uint16_t property,const char* value);
+getset_property_internal (PTPParams* params, uint16_t property,const char* value, short force);
 void
-getset_property_internal (PTPParams* params, uint16_t property,const char* value)
+getset_property_internal (PTPParams* params, uint16_t property,const char* value, short force)
 {
 	PTPDevicePropDesc dpd;
 	const char* propname;
@@ -1396,7 +1535,7 @@ getset_property_internal (PTPParams* params, uint16_t property,const char* value
 
 	memset(&dpd,0,sizeof(dpd));
 	result=ptp_getdevicepropdesc(params,property,&dpd);
-	if (result!=PTP_RC_OK) {
+	if (result!=PTP_RC_OK&&!force) {
 		ptp_perror(params,result); 
 		fprintf(stderr,"ERROR: "
 		"Could not get device property description!\n"
@@ -1580,10 +1719,11 @@ getset_propertybyname (int busn,int devn,char* property,char* value,short force)
 		if (propval==NULL) propval=value;
 	}
 
-	getset_property_internal (&params, dpc,propval);
+	getset_property_internal (&params, dpc,propval, force);
 
 	close_camera(&ptp_usb, &params, dev);
 }
+
 void
 getset_property (int busn,int devn,uint16_t property,char* value,short force);
 void
@@ -1603,125 +1743,76 @@ getset_property (int busn,int devn,uint16_t property,char* value,short force)
 		printf(" (bus %i, dev %i)\n",busn,devn);
 	else
 		printf("\n");
-	if (!ptp_property_issupported(&params, property))
+	if (!ptp_property_issupported(&params, property)&&!force)
 	{
 		fprintf(stderr,"The device does not support this property!\n");
 		close_camera(&ptp_usb, &params, dev);
 		return;
 	}
 
-	getset_property_internal (&params, property,value);
-#if 0
-	memset(&dpd,0,sizeof(dpd));
-	CR(ptp_getdevicepropdesc(&params,property,&dpd),
-		"Could not get device property description!\n"
-		"Try to reset the camera.\n");
-	propdesc= ptp_prop_getdesc(&params, &dpd, NULL);
-	propname=ptp_prop_getname(&params,property);
-
-	if (value==NULL) { /* property GET */
-		if (!verbose) { /* short output, default */
-			printf("'%s' is set to: ", propname==NULL?"UNKNOWN":propname);
-			if (propdesc!=NULL)
-				printf("%s [%s]", ptp_prop_tostr(&params, &dpd,
-							NULL), propdesc);
-			else 
-			{
-				if (dpd.FormFlag==PTP_DPFF_Enumeration)
-					PRINT_PROPVAL_HEX(dpd.CurrentValue);
-				else 
-					PRINT_PROPVAL_DEC(dpd.CurrentValue);
-			}
-			printf("\n");
-	
-		} else { /* verbose output */
-	
-			printf("%s: [0x%04x, ",propname==NULL?"UNKNOWN":propname,
-					property);
-			if (dpd.GetSet==PTP_DPGS_Get)
-				printf ("readonly, ");
-			else
-				printf ("readwrite, ");
-			printf ("%s] ",
-				ptp_get_datatype_name(&params, dpd.DataType));
-
-			printf ("\n  Current value: ");
-			if (dpd.FormFlag==PTP_DPFF_Enumeration)
-				PRINT_PROPVAL_HEX(dpd.CurrentValue);
-			else 
-				PRINT_PROPVAL_DEC(dpd.CurrentValue);
-
-			if (propdesc!=NULL)
-				printf(" [%s]", propdesc);
-			printf ("\n  Factory value: ");
-			if (dpd.FormFlag==PTP_DPFF_Enumeration)
-				PRINT_PROPVAL_HEX(dpd.FactoryDefaultValue);
-			else 
-				PRINT_PROPVAL_DEC(dpd.FactoryDefaultValue);
-			propdesc=ptp_prop_getdesc(&params, &dpd,
-						dpd.FactoryDefaultValue);
-			if (propdesc!=NULL)
-				printf(" [%s]", propdesc);
-			printf("\n");
-
-			switch (dpd.FormFlag) {
-			case PTP_DPFF_Enumeration:
-				{
-					int i;
-					printf ("Enumerated:\n");
-					for(i=0;i<dpd.FORM.Enum.NumberOfValues;i++){
-						PRINT_PROPVAL_HEX(
-						dpd.FORM.Enum.SupportedValue[i]);
-						propdesc=ptp_prop_getdesc(&params, &dpd, dpd.FORM.Enum.SupportedValue[i]);
-						if (propdesc!=NULL) printf("\t[%s]", propdesc);
-						printf("\n");
-					}
-				}
-				break;
-			case PTP_DPFF_Range:
-				printf ("Range [");
-				PRINT_PROPVAL_DEC(dpd.FORM.Range.MinimumValue);
-				printf(" - ");
-				PRINT_PROPVAL_DEC(dpd.FORM.Range.MaximumValue);
-				printf("; step ");
-				PRINT_PROPVAL_DEC(dpd.FORM.Range.StepSize);
-				printf("]\n");
-				break;
-			case PTP_DPFF_None:
-				break;
-			}
-		}
-	} else {
-		uint16_t r;
-		propdesc= ptp_prop_getdesc(&params, &dpd, NULL);
-		printf("'%s' is set to: ", propname==NULL?"UNKNOWN":propname);
-		if (propdesc!=NULL)
-			printf("%s [%s]", ptp_prop_tostr(&params, &dpd, NULL), propdesc);
-		else
-		{
-			if (dpd.FormFlag==PTP_DPFF_Enumeration)
-				PRINT_PROPVAL_HEX(dpd.CurrentValue);
-			else 
-				PRINT_PROPVAL_DEC(dpd.CurrentValue);
-		}
-		printf("\n");
-		printf("Changing property value to '%s' ",value);
-		r=(set_property(&params, property, value, dpd.DataType));
-		if (r!=PTP_RC_OK)
-		{
-			printf ("FAILED!!!\n");
-			fflush(NULL);
-		        ptp_perror(&params,r);
-		}
-		else 
-			printf ("succeeded.\n");
-	}
-	ptp_free_devicepropdesc(&dpd);
-#endif
+	getset_property_internal (&params, property,value, force);
 
 	close_camera(&ptp_usb, &params, dev);
 }
 
+void
+getset_property_value (int busn,int devn,uint16_t property,char* value,short force);
+void
+getset_property_value (int busn,int devn,uint16_t property,char* value,short force)
+{
+	PTPParams params;
+	PTP_USB ptp_usb;
+	struct usb_device *dev;
+
+	printf ("\n");
+
+	if (open_camera(busn, devn, force, &ptp_usb, &params, &dev)<0)
+		return;
+
+	printf("Camera: %s",params.deviceinfo.Model);
+	if ((devn!=0)||(busn!=0)) 
+		printf(" (bus %i, dev %i)\n",busn,devn);
+	else
+		printf("\n");
+	if (!ptp_property_issupported(&params, property)&&!force)
+	{
+		fprintf(stderr,"The device does not support this property!\n");
+		close_camera(&ptp_usb, &params, dev);
+		return;
+	}
+
+	
+
+/* Transaction data phase description */
+#define PTP_DP_NODATA		0x0000	/* no data phase */
+#define PTP_DP_SENDDATA		0x0001	/* sending data */
+#define PTP_DP_GETDATA		0x0002	/* receiving data */
+#define PTP_DP_DATA_MASK	0x00ff	/* data phase mask */
+#define PTP_CNT_INIT(cnt) {memset(&cnt,0,sizeof(cnt));}
+	PTPContainer ptp;
+	uint16_t ret;
+	char* dpv=NULL;
+
+	PTP_CNT_INIT(ptp);
+	ptp.Code=PTP_OC_GetDevicePropValue;
+	ptp.Param1=property;
+	ptp.Nparam=1;
+	ret=ptp_transaction(&params, &ptp, PTP_DP_GETDATA, 0, &dpv);
+	//result =ptp_getdevicepropvalue (&params, property, &prop, PTP_DTC_UINT8);
+
+	if (ret!=PTP_RC_OK) {
+		ptp_perror(&params,ret); 
+		fprintf(stderr,"ERROR: "
+		"Could not get device property description!\n"
+		"Try to reset the camera.\n");
+		if (!force) return ;
+	}
+	if (dpv) printf("%x is set to: %08x\n", property, *((uint8_t*)dpv));
+
+	close_camera(&ptp_usb, &params, dev);
+}
+
+
 void
 show_all_properties (int busn,int devn,short force, int unknown);
 void
@@ -1751,7 +1842,7 @@ show_all_properties (int busn,int devn,short force, int unknown)
 				params.deviceinfo.DevicePropertiesSupported[i]);
 		if ((unknown) && (propname!=NULL)) continue;
 
-		printf("0x%04x: ",
+		printf("  0x%04x: ",
 				params.deviceinfo.DevicePropertiesSupported[i]);
 		memset(&dpd,0,sizeof(dpd));
 		CR(ptp_getdevicepropdesc(&params,
@@ -1760,14 +1851,14 @@ show_all_properties (int busn,int devn,short force, int unknown)
 			"Try to reset the camera.\n");
 		propdesc= ptp_prop_getdesc(&params, &dpd, NULL);
 
-		PRINT_PROPVAL_HEX(dpd.CurrentValue);
 		if (verbose) {
-			printf (" (%s",propname==NULL?"UNKNOWN":propname);
+			printf ("(%s",propname==NULL?"UNKNOWN":propname);
 			if (propdesc!=NULL)
-				printf(": %s)",propdesc);
+				printf(": %s) ",propdesc);
 			else
-				printf(")");
+				printf(") ");
 		}
+		PRINT_PROPVAL_DEC(dpd.CurrentValue);
 	
 		printf("\n");
 		ptp_free_devicepropdesc(&dpd);
@@ -1893,6 +1984,67 @@ reset_device (int busn, int devn, short force)
 
 }
 
+
+uint16_t
+ptp_transaction_nodata (PTPParams* params, PTPContainer* ptp);
+uint16_t
+ptp_transaction_nodata (PTPParams* params, PTPContainer* ptp)
+{
+	return (ptp_transaction(params, ptp, PTP_DP_NODATA, 0, 0));
+}
+
+uint16_t
+ptp_transaction_getdata (PTPParams* params, PTPContainer* ptp, unsigned int *getlen, char** data);
+uint16_t
+ptp_transaction_getdata (PTPParams* params, PTPContainer* ptp, unsigned int *getlen, char** data)
+{
+	return (ptp_transaction(params, ptp, PTP_DP_GETDATA, (unsigned int) getlen, data));
+}
+
+uint16_t
+ptp_transaction_senddata (PTPParams* params, PTPContainer* ptp, unsigned int sendlen, char* data);
+uint16_t
+ptp_transaction_senddata (PTPParams* params, PTPContainer* ptp, unsigned int sendlen, char* data)
+{
+	return (ptp_transaction(params, ptp, PTP_DP_SENDDATA, sendlen, &data));
+}
+
+void ptphack()
+{
+	PTPParams params={};
+	PTP_USB ptp_usb={};
+	struct usb_device *dev;
+	PTPContainer ptp={};
+	int getlen=0;
+
+	char* data=NULL;
+
+	PTPDeviceInfo 	PTPDeviceInfo={};
+	PTPStorageIDs 	PTPStorageIDs={};
+	PTPStorageInfo 	PTPStorageInfo={};
+	PTPObjectHandles 	PTPObjectHandles={};
+	PTPObjectInfo	PTPObjectInfo={};
+	PTPPropDescRangeForm 	PTPPropDescRangeForm={};
+	PTPPropDescEnumForm 	PTPPropDescEnumForm={};
+	PTPDevicePropDesc 	PTPDevicePropDesc={};
+	PTPCANONFolderEntry 	PTPCANONFolderEntry={};
+
+	if (open_camera(0, 0, 0, &ptp_usb, &params, &dev)<0)
+		return;
+
+	raise(SIGINT);
+
+	//ptp_transaction(&params, &ptp, PTP_DP_GETDATA, 0, &dpv);
+	/*
+	 * ptp_transaction_nodata(&params, &ptp)
+	 * ptp_transaction_getdata(&params, &ptp, &getlen, &data)
+	 * ptp_transaction_senddata(&params, &ptp, sendlen, data)
+	 *
+	 */
+	
+	close_camera(&ptp_usb, &params, dev);
+}
+
 /* main program  */
 
 int
@@ -1908,6 +2060,11 @@ main(int argc, char ** argv)
 	uint32_t handle=0;
 	char *filename=NULL;
 	int num=0;
+	int interval=0;
+	uint16_t reqCode=0;
+	uint32_t reqParams[5];
+	uint32_t direction=PTP_DP_GETDATA;
+	char data_file[256];
 	/* parse options */
 	int option_index = 0,opt;
 	static struct option loptions[] = {
@@ -1934,6 +2091,7 @@ main(int argc, char ** argv)
 		{"nikon-dc2",0,0,0},
 		{"ndc2",0,0,0},
 		{"loop-capture",1,0,0},
+		{"interval",1,0,0},
 		{"delete-object",1,0,'d'},
 		{"delete-all-files",1,0,'D'},
 		{"info",0,0,'i'},
@@ -1949,7 +2107,7 @@ main(int argc, char ** argv)
 	signal(SIGINT, ptpcam_siginthandler);
 	
 	while(1) {
-		opt = getopt_long (argc, argv, "LhlcipfroGg:Dd:s:v::", loptions, &option_index);
+		opt = getopt_long (argc, argv, "LhlcipfroGg:Dd:s:v::R:", loptions, &option_index);
 		if (opt==-1) break;
 	
 		switch (opt) {
@@ -1979,6 +2137,8 @@ main(int argc, char ** argv)
 				propstr=strdup(optarg);
 				action=ACT_SET_PROPBYNAME;
 			}
+			if (!(strcmp("interval",loptions[option_index].name)))
+				interval=strtol(optarg,NULL,10);
 			if (!strcmp("nikon-dc", loptions[option_index].name) ||
 			    !strcmp("ndc", loptions[option_index].name))
 			{
@@ -2047,6 +2207,22 @@ main(int argc, char ** argv)
 			break;
 		case 'D':
 			action=ACT_DELETE_ALL_FILES;
+			break;
+		case 'R':
+			action=ACT_GENERIC_REQ;
+			memset((void*)reqParams, 0, sizeof(uint32_t) * 5);
+			memset((void*)data_file, 0, 256);
+			sscanf(optarg, "%x,%x,%x,%x,%x,%x,%s", (uint*)&reqCode, &reqParams[0], &reqParams[1],
+				   &reqParams[2], &reqParams[3], &reqParams[4], data_file);
+			if (data_file[0] == 'r') {
+			  direction = PTP_DP_GETDATA;
+			} else if (data_file[0] == 'n') {
+			  direction = PTP_DP_NODATA;
+			} else if (strlen(data_file)>0) {
+			  direction = PTP_DP_SENDDATA;
+			  printf("data to send : '%s'\n", data_file);
+			}
+			break;
 		case '?':
 			break;
 		default:
@@ -2070,6 +2246,7 @@ main(int argc, char ** argv)
 			list_properties(busn,devn,force);
 			break;
 		case ACT_GETSET_PROPERTY:
+			//getset_property_value(busn,devn,property,value,force);
 			getset_property(busn,devn,property,value,force);
 			break;
 		case ACT_SHOW_INFO:
@@ -2084,6 +2261,9 @@ main(int argc, char ** argv)
 		case ACT_GET_FILE:
 			get_file(busn,devn,force,handle,filename,overwrite);
 			break;
+		case ACT_GENERIC_REQ:
+		    send_generic_request (busn, devn, reqCode, reqParams, direction, data_file);
+			break;
 		case ACT_GET_ALL_FILES:
 			get_all_files(busn,devn,force,overwrite);
 			break;
@@ -2097,7 +2277,7 @@ main(int argc, char ** argv)
 			delete_all_files(busn,devn,force);
 			break;
 		case ACT_LOOP_CAPTURE:
-			loop_capture (busn,devn,force,num,overwrite);
+			loop_capture (busn,devn,force,num,interval, overwrite);
 			break;
 		case ACT_SHOW_ALL_PROPERTIES:
 			show_all_properties(busn,devn,force,0);
